/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
The ISV profile component renders a venders profile with stats based on there
charms and bundles. You can only see this page is you have access.
*/

'use strict';

YUI.add('plans-usage', function () {

  juju.components.PlansUsage = React.createClass({
    propTypes: {
      d3: React.PropTypes.object.isRequired,
      dataset: React.PropTypes.array.isRequired
    },

    /**
      Get the initial state.
       @method getInitialState
      @returns {String} The intial state.
    */
    getInitialState: function getInitialState() {
      // The parent node for the SVG chart
      this.svg = [];
      // Margins to add framing to the charts
      this.margins = {
        top: 20,
        right: 20,
        bottom: 20,
        left: 50
      };
      // The yScale of the chart
      this.yScale = [];
      // The xScale of the chart
      this.xScale = [];

      return {
        d3Rendered: false
      };
    },

    componentDidMount: function componentDidMount() {
      this.renderChart();
      this.setState({ d3Rendered: true });
    },

    shouldComponentUpdate: function shouldComponentUpdate() {
      return !this.state.d3Rendered;
    },

    /**
      Plot a line on the graph
       @method plotLine
      @param {Array} The line data
    */
    plotLine: function plotLine(data) {
      var _this = this;

      var d3 = this.props.d3;
      var lineGen = d3.svg.line().x(function (d) {
        return _this.xScale(d.date);
      }).y(function (d) {
        return _this.yScale(d.value);
      });

      this.svg.append('path').attr('d', lineGen(data)).attr('stroke', 'teal').classed('line', true);
    },

    /**
      Render the base chart, including the axises.
       @method renderChart
    */
    renderChart: function renderChart() {
      var dataset = this.props.dataset;
      var d3 = this.props.d3;
      var w = 700;
      var h = 500;
      var ticks = dataset[0].length;

      var flattenedDataset = dataset.reduce(function (a, b) {
        return a.concat(b);
      });

      // Crete a charting domain based on the value set
      var x_domain = d3.extent(flattenedDataset, function (d) {
        return d.date;
      });

      // Crete a charting domain based on the value set
      var y_domain = d3.extent(flattenedDataset, function (d) {
        return d.value;
      });

      // Display date format (05/16)
      var date_format = d3.time.format('%m/%d');

      // Create an SVG container
      var vis = d3.select('.d3-chart-wrapper');
      this.svg = vis.append('svg').attr('width', w).attr('height', h).classed('d3-chart', true);

      // Define the y scale
      this.yScale = d3.scale.linear().domain(y_domain).range([h - this.margins.top, this.margins.bottom]);

      // Define the x scale
      this.xScale = d3.time.scale().domain(x_domain).range([this.margins.left, w - this.margins.left]);

      // Define the y axis
      var yAxis = d3.svg.axis().orient('left').scale(this.yScale).tickFormat(function (d) {
        return '$' + d;
      });

      // Define the x axis
      var xAxis = d3.svg.axis().orient('bottom').ticks(ticks).scale(this.xScale).tickFormat(date_format);

      // Draw y axis with labels and move in from the size by the amount of
      // padding
      this.svg.append('g').attr('transform', 'translate(0, ' + (h - this.margins.bottom) + ')').classed('axis', true).call(xAxis);

      // Draw x axis with labels and move to the bottom of the chart area
      this.svg.append('g').attr('transform', 'translate(' + this.margins.left + ', 0)').classed('axis', true).call(yAxis);

      // Loop through dataset to plot each line
      dataset.forEach(this.plotLine);
    },

    render: function render() {
      return React.createElement(
        'div',
        { className: 'twelve-col isv-profile__plan-usage' },
        React.createElement(
          'h4',
          null,
          'Plan usage'
        ),
        React.createElement('div', { className: 'd3-chart-wrapper' })
      );
    }
  });
}, '', {
  requires: []
});